﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using FARO.DeviceInterface;

namespace FaroTrackerDLL
{
    using FARO.DeviceInterface;
    public class FaroTrackerInteractor
    {
        //readonly static Tracker tkr = null;
        private static Tracker tkr = null;

        public enum Measurements { ForeGnd = 0, BackGnd = 1 };
        public static int MeasurementType = (int)Measurements.ForeGnd;

        TrackerConnectListener connectListener;
        TrackerCmdCompleteListener cmdCompleteListener;
        //TrackerStatuslistener statuslistener; Kalyan says we're not supposed to use this
        TrackerChangelistener changelistener;
        SearchCmdCompleteListener searchCmdCompleteListener;
        TrackerApplicationListener faroApplicationListener;
        ForegndCmdCompleteListener foregndCmdCompleteListener;
        TrackerMeasureDataListener measureDataListener;
        TrackerBkndMeasureDataListener bkndMeasureDataListener;
        ABusyListener busyListener;
        //TrackerVideoDataListener videoDataListener;

        //static FaroTrackerInteractor()
        //{
        //   //tkr = new Tracker("TrackerKeystone");
        //}

        public void ADLLMethod()
        {
            Debug.WriteLine("OK, dll called.");
        }

      public void InstantiateTrackerAndConnect()
        {
            //FARO.DeviceInterface.Cfg.setInstallDir(
            //   "C:\\Users\\Bruce\\Source\\Workspaces\\NRK\\Sandbox\\FaroTracker.netTypeLibTestApp\\FaroTrackerTestApp\\bin\\Debug\\TestFolder");

            //string path = Directory.GetCurrentDirectory();


            //FARO.DeviceInterface.Cfg.setInstallDir(path + "\\TestFolder");
            FARO.DeviceInterface.Cfg.setInstallDir("C:\\FaroJRE42");
            try
            {
                tkr = new Tracker("TrackerKeystone");

                tkr.setBlocking(true);
                tkr.connect("128.128.128.100", "user", "");
                tkr.setBlocking(false);

                //form.UpdateConnectedFlag(tkr.connected());
            }
            catch (FARO.DeviceInterface.TrackerException exp)
            {
                ShowException(exp.getText());
                return;
            }

            AddListeners();

            if (!tkr.initialized())
            {
                try
                {
                    tkr.setBlocking(true);
                    tkr.initializeSmart(false);
                    tkr.setBlocking(false);
                }
                catch (FARO.DeviceInterface.TrackerException exp)
                {
                    Debug.WriteLine(exp.Message);
                    return;
                }
            }
            else if (!tkr.motorsOn())
            {
                tkr.setBlocking(true);
                tkr.changeMotorState(true);
                tkr.setBlocking(false);
            }


            //FARO.DeviceInterface.SMRTargetType SMRTarget = new SMRTargetType(38.1);
            //tkr.changeTargetType(SMRTarget);

            tkr.setBlocking(true);
            //tkr.home(false);

            var avgFilt = new AverageFilter();
            var nullStart = new NullStartTrigger();
            var trig = new IntervalTrigger(0.1);

            var cfg = new MeasureCfg(1, avgFilt, nullStart, trig);
            tkr.setMeasureEventRate(1);
            tkr.startBkndMeasurePoint(cfg);

            tkr.setBlocking(false);
        }

            private void ShowException(String s)
              {
                  Debug.WriteLine(s);
                  //form.ShowMessageBox(s);
              }

                     public void GetAvailableApplications()
                  {
                      if (tkr == null)
                          return;

                      //JObjectArray is used to obtain an array of data. 
                      FARO.DeviceInterface.JObjectArray appsArray = null;

                      try
                      {
                          //This method returns the list of available applications that can be started
                          //through the driver.
                          appsArray = tkr.availableApplications();

                          if (appsArray != null)
                          {
                              Debug.WriteLine("Available Apps:");
                              String[] names = new String[appsArray.numElements];
                              for (int i = 0; i < appsArray.numElements; i++)
                              {
                                  names[i] = (String)appsArray.getElement(i);
                                  Debug.WriteLine(names[i]);
                              }
                          }
                      }
                      catch (FARO.DeviceInterface.TrackerException e)
                      {
                          ShowException(e.getText());
                      }
                  }
                     public void OnRunApplication(String appName)
                     {
                         //try
                         //{
                         //To start the application the name of the application and then if any parameters are
                         //required can be passed. Since we do not need to pass any parameters we are passing null.
                         //For eg: if you would like to remove the buttons of home and backsight on the Trackerpad
                         //then the  parameter to be passed can be like
                         //CString paramStr = "-bHome = off -bBacksight = off"and call the function as
                         //tkr.startApplicationFrame(appName, paramStr);

                         //If you would like specify units for length and Angle
                         //CString paramStr = "-uLength = millimeters, 1000.0 -uAngular = degrees, 57.29577951"
                         //and call the function as	tkr.startApplicationFrame(appName, paramStr)
                         //Please refer to the documentation(Interface User's Guide.rtf) for more information on this

                         //cmdCompleteListener.showCmdCompleteExp(false);
                         //tkr.startApplicationFrame(appName, "");
                         //}
                         //catch (FARO.DeviceInterface.TrackerException e)
                         //{
                         //   ShowException(e.getText());
                         //}
                     }

                     public bool AddListeners()
                     {
                         try
                         {
                             //connectListener = new TrackerConnectListener(this);
                             searchCmdCompleteListener = new SearchCmdCompleteListener(this);
                             measureDataListener = new TrackerMeasureDataListener(this);

                             //foregndCmdCompleteListener = new ForegndCmdCompleteListener(this);
                             bkndMeasureDataListener = new TrackerBkndMeasureDataListener(this);
                             cmdCompleteListener = new TrackerCmdCompleteListener(this);
                             faroApplicationListener = new TrackerApplicationListener(this);
                             busyListener = new ABusyListener(this);
                             //videoDataListener = new TrackerVideoDataListener(this);

                             tkr.setBlocking(true);


                             //tkr.addConnectListener(connectListener);

                             tkr.addBusyListener(busyListener);

                             tkr.addCommandCompleteListener(cmdCompleteListener);

                             tkr.addApplicationListener(faroApplicationListener);

                             tkr.addBkndMeasureDataListener(bkndMeasureDataListener);

                             tkr.addMeasureDataListener(measureDataListener);


                             tkr.startAsync();

                             tkr.setBlocking(false);
                         }
                         catch (FARO.DeviceInterface.TrackerException exp)
                         {
                             Debug.WriteLine(exp.getText());
                             return false;
                         }
                         catch (FARO.DeviceInterface.JException exp1)
                         {
                             String text = exp1.Message + "\nPlease check that the JRE folder and the jar files\nare installed, and restart the application.";
                             Debug.WriteLine(text);
                             return false;
                         }
                         catch (Exception ex)
                         {
                             Debug.WriteLine(ex.Message);
                             return false;
                         }

                         return true;
                     }

                     /*************************************************************************************************
                     * Function: RemoveListeners()
                     * Comments: When the user disconnects from the Tracker all these listeners are removed
                     *************************************************************************************************/
                     public void RemoveListeners()
        {
            tkr.removeConnectListener(connectListener);
            tkr.removeCommandCompleteListener(cmdCompleteListener);
            tkr.removeBkndMeasureDataListener(bkndMeasureDataListener);

            connectListener = null;
            searchCmdCompleteListener = null;
            foregndCmdCompleteListener = null;
            measureDataListener = null;
            bkndMeasureDataListener = null;
            cmdCompleteListener = null;
            busyListener = null;
            //videoDataListener = null;
        }

                     internal class TrackerConnectListener : FARO.DeviceInterface.ConnectListener
                     {
                         FaroTrackerInteractor parent = null;
                         public TrackerConnectListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void connectionChanged(FARO.DeviceInterface.ConnectEvent cEvent)
                         {
                             if (cEvent.isConnected())
                                 ;
                             //parent.SendHUDStatusMsg(hudStatusType.connected);
                             else
                                 //parent.SendHUDStatusMsg(hudStatusType.disconnected);
                                 ;
                         }
                     };

                     /*************************************************************************************************
                     * Function: commandComplete( CommandCompleteEvent* event )
                     * Fires: This listener fires when ever the tracker a command submitted to the tracker is completed.
                     * Event: The event tells whether the command is completed or not.
                     * Comments: We are using this listener for Asynchronous messages also.
                     *************************************************************************************************/

                     internal class TrackerCmdCompleteListener : CommandCompleteListener
                     {
                         FaroTrackerInteractor parent = null;
                         bool showExp = false;
                         public TrackerCmdCompleteListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public void showCmdCompleteExp(bool flag)
                         {
                             showExp = flag;
                         }

                         public override void commandComplete(FARO.DeviceInterface.CommandCompleteEvent cmdCompleteEvent)
                         {
                             //Whenever Tracker throws an exception the exception has to be rethrown to determine
                             //what type of exception Tracker has thrown.
                             //An example is shown at the end of this file of how different exceptions can be caught	                
                             TrackerException e = cmdCompleteEvent.getRelatedException();
                             if (e != null)
                             {
                                 String text = e.getText();
                                 try
                                 {
                                     // rethrow this exception as a derived class to get the appropriate catch block branching		
                                     throw e;
                                 }
                                 //This exception is thrown whenever the Tracker command has been successfully
                                 //aborted.
                                 catch (FARO.DeviceInterface.AbortedException e1)
                                 {
                                     text = e1.getText();
                                 }
                                 catch (FARO.DeviceInterface.TrackerException e2)
                                 {
                                     //If it is not an aborted exception it could be some other exception.
                                     text = e2.getText();
                                 }

                                 //If a command complete listener is added and a tracker application is started, 
                                 //the listener fires on every command regardless of whether the parent application or a 
                                 //tracker application executes the command. Some of the tracker applications such as CompIT 
                                 //and Diagnostics perform checks to determine what type of hardware is installed in the tracker.
                                 //These checks may generate errors, which CompIT and Diagnostics filter because it expects them.
                                 //However, because the listener is added the parent application also sees these errors.  
                                 //There are two possible workarounds:
                                 //1) Remove the command complete listener whenever a tracker application is started.  
                                 //Then when the application closes, add the command complete listener back in. 
                                 //2) Add a flag to the command complete listener as being done  that can be used to get whether the command complete 
                                 //listener handles the response when the listener fires.  The flag can be set when a tracker application 
                                 //is started and cleared when the application closes.

                                 //parent.SendGeneralStatusMsg(text);
                                 //listenerMode.ShowException(text);
                                 //If exception is not thrown it means that the command is successful.
                             }
                             //if (parent.eProcWaiting == procWaitStatus.pw_waiting)
                             //{
                             //    parent.eProcWaiting = procWaitStatus.pw_donesuccess;
                             //}
                         }
                     };

                     /*************************************************************************************************
                     * Function: statusChanged(StatusEvent* event)
                     * Fires: Whenever a status event occurs this listener fires.
                     * Event: The event tells which type of status message is generated.
                     * Comments:	Whenever the listener fires it can be checked for which status message it fired
                     *			correspondingly update the status on the status bar or notify the user about it etc.
                     *************************************************************************************************/

                     internal class TrackerStatuslistener : FARO.DeviceInterface.StatusListener
                     {
                         FaroTrackerInteractor parent = null;

                         public TrackerStatuslistener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void statusChanged(FARO.DeviceInterface.StatusEvent statusEvent)
                         {
                             FARO.DeviceInterface.Status status = statusEvent.getStatus();

                             int code = status.getCode();
                             //parent.statusCode = code;
                             //if motors are OFF it returns true else false
                             if (code == FARO.DeviceInterface.Status.MOTORS_OFF)
                             {
                                 bool state = status.getState();
                                 String[] s = new String[2];
                                 if (state == true)
                                 {
                                     //parent.SendHUDStatusMsg(hudStatusType.notReady);
                                     //parent.SendMeasurementStatusMsg("Motors Off");
                                     //parent.SendGeneralStatusMsg("Motors Off");
                                 }
                             }
                             //if distance reset is required it returns true else false
                             else if (code == FARO.DeviceInterface.Status.NEED_DISTANCE_RESET)
                             {
                                 bool valid = status.getState();
                                 if (valid)
                                 {
                                     ;
                                     //parent.SendHUDStatusMsg(hudStatusType.notReady);
                                     //parent.SendMeasurementStatusMsg("Not Ready");
                                     //parent.SendGeneralStatusMsg("Not Ready");

                                 }
                                 else
                                 {
                                     ;
                                     // if (!iProbe.isStablePtMeasuring)
                                     //{
                                     // blt - don't send during stable pt meas, it causes the beam/stable/meas 
                                     // notification graphic to flicker off ("ready" is a blank graphic)
                                     //parent.SendHUDStatusMsg(hudStatusType.ready);
                                     // }
                                     //parent.SendMeasurementStatusMsg("Ready");
                                     //parent.SendGeneralStatusMsg("Ready");
                                 }
                             }
                         }
                     };
                     /*************************************************************************************************
                     * Function: changeOccurred( ChangeEvent* event)
                     * Fires: Whenever a change event occurs this listeners fires.
                     * Event: change event
                     * Comments: Same as Status listener
                     *************************************************************************************************/

                     internal class TrackerChangelistener : FARO.DeviceInterface.ChangeListener
                     {
                         FaroTrackerInteractor parent = null;

                         public TrackerChangelistener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void changeOccurred(FARO.DeviceInterface.ChangeEvent changeEvent)
                         {
                             FARO.DeviceInterface.Change change = changeEvent.getChange();
                             int changeCode = change.getCode();

                             if (changeCode == FARO.DeviceInterface.Change.SIGHT_ORIENTATION)
                             {
                                 //this is not working yet will need to fix this issue.
                                 //FARO.DeviceInterface.JBoolean boolvalue = (FARO.DeviceInterface.JBoolean)change.getRelatedData();
                                 //bool sightMode = boolvalue.getBoolean();
                                 bool sightMode = false;

                                 //if sightMode is true then tracker is in backsight else in frontsight.
                                 if (sightMode)
                                 {
                                     //parent.SendMeasurementStatusMsg("In Back Sight");
                                 }
                                 else
                                 {
                                     //parent.SendMeasurementStatusMsg("In Front Sight");
                                 }
                             }
                             else if (changeCode == FARO.DeviceInterface.Change.TARGET_TYPE)
                             {
                                 FARO.DeviceInterface.TargetType target = ((FARO.DeviceInterface.TargetType)change.getRelatedData());

                                 if (target is FARO.DeviceInterface.SMRTargetType)
                                 {
                                     FARO.DeviceInterface.SMRTargetType SMRTarget = (FARO.DeviceInterface.SMRTargetType)(target);
                                     if (SMRTarget != null)
                                     {
                                         double diameter = SMRTarget.getDiameter();

                                         //for (int i = 0; i < parent.targetDiameters.Length; i++)
                                         //{
                                         //    if (parent.targetDiameters[i] == diameter)
                                         //    {
                                         //        parent.selectedTargetIdx = i;
                                         //        iProbe.OnTipChange(diameter / 2.0, parent.targetTypes[i], true, targAtt);
                                         //        break;
                                         //    }
                                         //}
                                     }
                                 }
                                 else if (target is FARO.DeviceInterface.MirrorTargetType)
                                 {
                                     FARO.DeviceInterface.MirrorTargetType mirrorTarget = (FARO.DeviceInterface.MirrorTargetType)(target);
                                     if (mirrorTarget != null)
                                     {
                                         //parent.selectedTargetIdx = 3;
                                         //iProbe.OnTipChange(0.0, "SMM", true, targAtt);
                                     }
                                 }
                             }
                         }
                     };

                     /*************************************************************************************************
                     * Function: commandComplete( CommandCompleteEvent* event )
                     * Fires: This listener fires when ever the tracker has completed executing Tracker command.
                     * Event: The event tells if the Tracker has thrown an exception or if a command has 
                     *		 executed sucessfully.
                     *************************************************************************************************/

                     internal class SearchCmdCompleteListener : FARO.DeviceInterface.CommandCompleteListener
                     {
                         FaroTrackerInteractor parent = null;
                         public SearchCmdCompleteListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void commandComplete(FARO.DeviceInterface.CommandCompleteEvent cmdCompleteEvent)
                         {
                             //Whenever Tracker throws an exception the exception has to be rethrown to determine
                             //what type of exception Tracker has thrown.
                             //An example is shown at the end of this file of how different exceptions can be caught
                             FARO.DeviceInterface.TrackerException e = cmdCompleteEvent.getRelatedException();
                             String text = e.getText();
                             try
                             {
                                 // rethrow this exception as a derived class to get the appropriate catch block branching		
                                 throw e;
                             }
                             //This exception is thrown whenever the Tracker command has been successfully
                             //aborted.
                             catch (FARO.DeviceInterface.AbortedException e1)
                             {
                                 text = e1.getText();
                                 // parent.SendMeasurementStatusMsg(text);
                             }
                             catch (FARO.DeviceInterface.TrackerException e2)
                             {
                                 //If it is not an aborted exception it could be some other exception.
                                 text = e2.getText();
                                 //parent.SendMeasurementStatusMsg(text);
                             }


                             //If exception is not thrown it means that the command is successful.	
                             //LPARAM lParam = (LPARAM)text;
                             //AfxGetMainWnd()->PostMessage(UWM_SEARCH_CMD_FIRED, lParam, 0);	
                             //PostMessage(hWND, WM_COMMAND, IDOK, 0L);

                         }
                     };

                     /*************************************************************************************************
                     * Function: stateChanged(ApplicationEvent* event)
                     * Fires: This listener fires when ever an application is started are stopped.
                     * Event: The event tells which application has started and whether it has started or stopped
                     *************************************************************************************************/

                     internal class TrackerApplicationListener : FARO.DeviceInterface.ApplicationListener
                     {
                         FaroTrackerInteractor parent = null;
                         public TrackerApplicationListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void stateChanged(FARO.DeviceInterface.ApplicationEvent appEvent)
                         {
                             String name = appEvent.getAppName();
                             bool run = appEvent.isAppRunning();

                             //parent.SendGeneralStatusMsg(name);
                         }
                     };

                     /*************************************************************************************************
                     * Function: commandComplete( CommandCompleteEvent* event )
                     * Listener: Command Complete Listener
                     * Comments: When the measurement has been completely stopped this listener fires
                     *************************************************************************************************/

                     internal class ForegndCmdCompleteListener : FARO.DeviceInterface.CommandCompleteListener
                     {
                         FaroTrackerInteractor parent = null;
                         public ForegndCmdCompleteListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void commandComplete(FARO.DeviceInterface.CommandCompleteEvent cmdCompleteEvent)
                         {
                             FARO.DeviceInterface.JObject obj = cmdCompleteEvent.getSource();

                             FARO.DeviceInterface.TrackerException exp = cmdCompleteEvent.getRelatedException();
                             if (exp != null)
                             {
                                 // parent.SendGeneralStatusMsg(exp.getText());
                             }
                             else
                             {
                                 //AfxGetMainWnd()->PostMessage(UWM_FOREGND_MEASUREMENT_STOPPED);
                                 tkr.removeCommandCompleteListener(parent.foregndCmdCompleteListener);
                             }
                         }
                     };

                     /*************************************************************************************************
                     * Function: dataAvailable( MeasureDataEvent* event )
                     * Listener: MeasureData Listener fires when the data is available after the measurements are started
                     * Event:	Event returns the measured data.
                     * Comments: The listener fires when all the no of records set by the measure event rate are available.
                                 Once the listener fires all the data can be retrieved.
                                 If the measure event rate is set for more than one observation then the data is returned
                                 in the form of SMXObjectArray. An example of this is also shown.
                     *************************************************************************************************/

                     internal class TrackerMeasureDataListener : FARO.DeviceInterface.MeasureDataListener
                     {
                         FaroTrackerInteractor parent = null;

                         public TrackerMeasureDataListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                             //iProbe = parent._probing as FaroTrackerProbing;
                         }

                         public override void dataAvailable(FARO.DeviceInterface.MeasureDataEvent dataEvent)
                         {

                             //Here we are using JObjectArray for all the cases like
                             //If the user wants to read a single point of data or 10 points of data or just
                             //keep on collecting data untill the user selects stop.
                             //This is just being done to provide an example that SMXObjectArray can be used in all cases.
                             //Where as in blocking mode and in worker thread mode for single point data
                             //we are using readmeasurepointdata to provide an example that if single point data
                             //is required it can be done in that way and for multiple data this way.

                             FARO.DeviceInterface.JObjectArray dataArray = null;
                             FARO.DeviceInterface.MeasurePointData mData = null;

                             //if(ptsWanted>0)
                             //{
                             //    if(ptsGathered >= ptsWanted)
                             //    {
                             //        //TRACE0("LeicaLMFLowLevel::ProcessLMFMeas, returning, already have all pts.\n");
                             //        return;
                             //    }
                             //}

                             //double planarOffset = iProbe.GetTipRad();
                             //double radialOffset = iProbe.GetTipRad();

                             //Measurement saMeas = new Measurement();
                             //saMeas.planarOffset = planarOffset;
                             //saMeas.radialOffset = radialOffset;

                             try
                             {
                                 int rate = tkr.getMeasureEventRate();
                                 dataArray = tkr.readMeasurePointData(rate);
                                 if (dataArray != null)
                                 {
                                     int numElements = dataArray.numElements;
                                     if (numElements > 0)
                                     {
                                         //Debug.WriteLine("3d scan Info: " + dataArray.);
                                         for (int i = 0; i < numElements; i++)
                                         {
                                             mData = (FARO.DeviceInterface.MeasurePointData)dataArray.getElement(i);
                                             if (mData != null)
                                             {
                                                 //status[i] = mData.status();

                                                 //NVectorSpheric posDHV = new NVectorSpheric();

                                                 //posDHV.r = mData.distance() * 1000.0;
                                                 //posDHV.theta = 180.0 + mData.azimuth() / Globals.DegToRad;
                                                 //posDHV.phi = mData.zenith() / Globals.DegToRad;

                                                 //if(!isInFrontFace) // Need Face 2 obs from Leica trackers (so SA can tell the difference)
                                                 //{
                                                 //    posDHV.y -= 180.0;
                                                 //    posDHV.z = 360.0 - posDHV.z;
                                                 //}

                                                 var timeStamp = mData.time();
                                                 Debug.WriteLine("Meas Time Stamp: {0}sec", timeStamp);

                                                 //if (parent.resetTimeStampOffset)
                                                 //{
                                                 //    parent.resetTimeStampOffset = false;
                                                 //    parent.timeStampOffset = timeStamp;
                                                 //}

                                                 //CriticalSectionLock lock( scannerCS );
                                                 //pts = tData.AddPt(posDHV, timeStamp);
                                                 //ptsGathered++;
                                                 ////TRACE0("ISingleShotMeasurement3DCOMPtr data added to tdata:\n");
                                                 ////posDHV.Debug();

                                                 //if(ptsWanted>0)
                                                 //{
                                                 //    if(ptsGathered >= ptsWanted)
                                                 //    {
                                                 //        TRACE0("LeicaLMFLowLevel::ProcessLMFMeas, stopping measurement.\n");
                                                 //        wm.KillTimer(SCANPTSTIMER);
                                                 //        StopMeasurement();
                                                 //        wm.PostMessage(SEND_DATA,0,0);
                                                 //        return;
                                                 //    }
                                                 //}

                                                 //if(pts >= 1000)
                                                 //{
                                                 //	TRACE1(">>>>>>sending %d pts\n", pts);
                                                 //	wm.PostMessage(SEND_DATA,0,0);
                                                 //	ProcessWaitingMessages();
                                                 //	pts = 0;
                                                 //}
                                                 //TRACE0("ISingleShotMeasurement3DCOMPtr data added to tdata:\n");
                                                 //posDHV.Debug();

                                                 //NUnitVector approach = MeasurementUtilities.GetApproachVector(posDHV.GetCartesian(), 10.0);
                                                 //saMeas.Add3dProbeMeas(posDHV.GetCartesian(), approach);
                                             }
                                             //iProbe.OnMeasurement(saMeas);
                                             //var currentAc = iProbe?.GetAcquisitionProfiles()?.GetActionByName(iProbe?.GetCurrentAcquisitionProfile());
                                             //if (currentAc?.acType == InstrumentAction.AcType.SinglePt || currentAc?.acType == InstrumentAction.AcType.StablePt)
                                             //{
                                             //    //iProbe.StopMeasure(currentAc.name);
                                             //    parent.StopMeasuring();
                                             //}
                                         }

                                         //if (Globals.MeasurementType == (int)Globals.Measurements.ForeGnd)
                                         //{
                                         //    listenerMode.UpdateForeGndMeasurementArray(az, ze, dist, status);
                                         //}
                                         //else
                                         //{
                                         //    listenerMode.UpdateBackGndMeasurementArray(az, ze, dist, status);
                                         //}
                                     }
                                 }
                             }
                             catch (FARO.DeviceInterface.TrackerException e)
                             {
                                 //parent.SendGeneralStatusMsg(e.getText());
                             }
                         }
                     };

                     /*************************************************************************************************
                     * Function: dataAvailable( BkndMeasureDataEvent* event )
                     * Listener: Uses Background measure data listener
                     * Event:	Returns the data
                     * Comments: This listener fires when ever the data is available once the measurements are started.
                     *************************************************************************************************/

                     internal class TrackerBkndMeasureDataListener : BkndMeasureDataListener
                     {
                         FaroTrackerInteractor parent = null;

                         public TrackerBkndMeasureDataListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void dataAvailable(BkndMeasureDataEvent dataEvent)
                         {
                             if (tkr == null)//|| !parent.isConnected)
                                 return;

                             FARO.DeviceInterface.JObjectArray dataArray = null;
                             FARO.DeviceInterface.MeasurePointData mData = null;

                             try
                             {
                                 int rate = tkr.getMeasureEventRate();
                                 dataArray = tkr.readBkndMeasurePointData(rate);
                                 if (dataArray != null)
                                 {
                                     int numElements = dataArray.numElements;
                                     if (numElements > 0)
                                     {
                                         //Debug.WriteLine("3d scan Info: " + dataArray.);
                                         for (int i = 0; i < numElements; i++)
                                         //int i = numElements - 1;
                                         {
                                             mData = (FARO.DeviceInterface.MeasurePointData)dataArray.getElement(i);
                                             if (mData != null)
                                             {
                                                 Debug.WriteLine("Bkgnd Time Stamp: {0} sec", mData.time());
                                                 Debug.WriteLine("Bkgnd Pos: r= {0}, az={1}, zen={2}", mData.distance(), mData.azimuth(), mData.zenith());
                                             }
                                             else if (mData.status() == FARO.DeviceInterface.MeasurePointData.DATA_INACCURATE || mData.status() == FARO.DeviceInterface.MeasurePointData.DATA_ERROR)
                                             {
                                                 Debug.WriteLine("Data Inaccurate or Data Error", mData.time());
                                             }
                                         }
                                     }
                                 }
                             }
                             catch (FARO.DeviceInterface.TrackerException e)
                             {
                                 Debug.WriteLine(e.getText());
                             }
                         }
                     };

                     /////////////////////////////////////////////////////////////////////
                     internal class ABusyListener : BusyListener
                     {
                         private FaroTrackerInteractor parent = null;

                         public ABusyListener(FaroTrackerInteractor _parent)
                         {
                             this.parent = _parent;
                         }

                         public override void stateChanged(BusyEvent evt)
                         {
                             if (evt.getMeasureState())
                             {
                                 //parent.isMeasuring = true;
                             }
                             else
                             {
                                 //parent.isMeasuring = false;
                             }
                         }
                     };
    }
}
